/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { AttachmentContainerE2EComponent } from 'apps/alfa-e2e/src/components/attachment/attachment.e2e.component';
import { VorgangFormularDatenE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-formular.e2e.component';
import { convertToDataTestId } from 'apps/alfa-e2e/src/support/tech.util';
import { AttachmentListE2EComponent } from '../../../components/attachment/attachment.e2e.component';
import { VorgangDetailHeaderE2EComponent } from '../../../components/vorgang/vorgang-detail-header.e2e.component';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { VorgangSubnavigationE2EComponent } from '../../../components/vorgang/vorgang-subnavigation';
import { FileDataE2E } from '../../../model/binary-file';
import { EingangE2E, VorgangE2E } from '../../../model/vorgang';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import {
  createJpgAttachment,
  createJpgGridFsData,
  createPdfAttachment,
  createPdfGridFsData,
  createXmlGridFsData,
  createXmlRepresentation,
  initGridFs,
} from '../../../support/binary-file-util';
import {
  countDownloadFiles,
  deleteDownloadFolder,
  dropCollections,
  getDownloadFiles,
  readFileFromDownloads,
  unzipDownloadFile,
} from '../../../support/cypress-helper';
import { exist, notExist } from '../../../support/cypress.util';
import { loginAsSabine } from '../../../support/user-util';
import {
  buildVorgang,
  createVorgang,
  initVorgaenge,
  objectIds,
} from '../../../support/vorgang-util';

describe('Vorgang Anhänge', () => {
  const mainPage: MainPage = new MainPage();
  const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

  const vorgangPage: VorgangPage = new VorgangPage();
  const vorgangHeader: VorgangDetailHeaderE2EComponent = vorgangPage.getVorgangDetailHeader();
  const subnavigation: VorgangSubnavigationE2EComponent = vorgangPage.getSubnavigation();
  const attachmentContainer: AttachmentContainerE2EComponent = vorgangPage.getAttachmentContainer();
  const attachmentList: AttachmentListE2EComponent = new AttachmentListE2EComponent();
  const formularDatenContainer: VorgangFormularDatenE2EComponent =
    vorgangPage.getFormularDatenContainer();
  const vorgangDatenFormular: VorgangFormularDatenE2EComponent =
    vorgangPage.getFormularDatenContainer();

  const xmlRepresentation: FileDataE2E = createXmlRepresentation();

  const sonstigeAttachment = {
    name: 'datei_sonstiges',
    files: [createJpgAttachment(), createPdfAttachment()],
  };
  const vorgangKeinAnhang: VorgangE2E = createVorgang();

  const eingangWithAttachments: EingangE2E = {
    ...createVorgang().eingangs[0],
    numberOfRepresentations: 1,
    representations: [xmlRepresentation],
    numberOfAttachments: 2,
    attachments: [sonstigeAttachment],
  };
  const vorgangAnhangVorhanden: VorgangE2E = {
    ...buildVorgang(objectIds[0], 'VorgangWithAnhang'),
    eingangs: [eingangWithAttachments],
  };

  before(() => {
    initGridFs([createXmlGridFsData(), createPdfGridFsData(), createJpgGridFsData()]);
    initVorgaenge([vorgangKeinAnhang, vorgangAnhangVorhanden]);

    loginAsSabine();

    waitForSpinnerToDisappear();
    exist(vorgangList.getRoot());
  });

  after(() => {
    dropCollections();
  });

  describe('Download Attachment', () => {
    it('should open vorgang detail page on click on single vorgang', () => {
      vorgangList.getListItem(vorgangAnhangVorhanden.name).getRoot().click();
      waitForSpinnerToDisappear();

      exist(vorgangHeader.getRoot());
    });

    it('should show attachment on click on tab', () => {
      formularDatenContainer.getDateienTab().click();

      notExist(vorgangPage.getSpinner());
      exist(attachmentContainer.getList().getRoot());
    });

    it('should download jpg', () => {
      const jpgAttachmentName: string = getJpgAttachment().name;
      attachmentContainer.getList().getItem(jpgAttachmentName).getRoot().click();
      waitForSpinnerToDisappear();

      exist(readFileFromDownloads(buildDownloadFileName(jpgAttachmentName)));
    });

    it('should download pdf', () => {
      const pdfAttachmentName: string = getPdfAttachment().name;
      attachmentContainer
        .getList()
        .getItem(convertToDataTestId(pdfAttachmentName))
        .getRoot()
        .click();
      waitForSpinnerToDisappear();

      exist(readFileFromDownloads(buildDownloadFileName(pdfAttachmentName)));
    });

    it('should download attachment zip file', () => {
      deleteDownloadFolder().then(() => {
        attachmentList.downloadAttachments();
      });
    });

    it('should unzip attachment file', () => {
      getDownloadFiles().then((files) => {
        unzipDownloadFile(files[0]).then(() => {
          countDownloadFiles().then((count) => {
            expect(count).to.eq(3);
          });
        });
      });
    });

    it('should navigate to vorganglist on back button', () => {
      subnavigation.getBackButton().click();
      waitForSpinnerToDisappear();

      exist(vorgangList.getRoot());
    });

    function getJpgAttachment(): FileDataE2E {
      return sonstigeAttachment.files[1];
    }

    function getPdfAttachment(): FileDataE2E {
      return sonstigeAttachment.files[0];
    }
  });

  describe('Download Representation', () => {
    it('should open vorgang detail page on click on single vorgang', () => {
      vorgangList.getListItem(vorgangAnhangVorhanden.name).getRoot().click();
      waitForSpinnerToDisappear();

      exist(vorgangHeader.getRoot());
    });

    it('should show attachment on click on tab', () => {
      formularDatenContainer.getDateienTab().click();

      notExist(vorgangPage.getSpinner());
      exist(attachmentContainer.getList().getRoot());
    });

    it('should download representation', () => {
      attachmentContainer.getList().getItem(xmlRepresentation.name).getRoot().click();
      waitForSpinnerToDisappear();

      exist(readFileFromDownloads(buildDownloadFileName(xmlRepresentation.name)));
    });

    it('should navigate to vorganglist on back button', () => {
      subnavigation.getBackButton().click();
      waitForSpinnerToDisappear();

      exist(vorgangList.getRoot());
    });
  });

  function buildDownloadFileName(fileName: string): string {
    return `${getFileNamePrefix()}_${fileName}`;
  }

  function getFileNamePrefix(): string {
    return vorgangAnhangVorhanden.nummer;
  }
});
