import { createEmptyStateResource } from '@alfa-client/tech-shared';
import {
  dispatchEventFromFixture,
  getDebugElementFromFixtureByCss,
  getElementFromFixture,
  mock,
  Mock,
} from '@alfa-client/test-utils';
import { DebugElement } from '@angular/core';
import { ComponentFixture, fakeAsync, TestBed, tick } from '@angular/core/testing';
import { AbstractControl, FormsModule, ReactiveFormsModule, UntypedFormGroup } from '@angular/forms';
import { ButtonWithSpinnerComponent } from '@ods/component';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent, ngMocks } from 'ng-mocks';
import { of, throwError } from 'rxjs';
import { createOrganisationseinheit } from '../../../../../test/user/user';
import { TextFieldComponent } from '../../../shared/text-field/text-field.component';
import { Organisationseinheit } from '../../../user/user.model';
import { OrganisationseinheitService } from '../../organisationseinheit.service';
import { OrganisationseinheitFormComponent } from './organisationseinheit-form.component';
import { OrganisationseinheitFormService } from './organisationseinheit-form.service';

describe('OrganisationseinheitFormComponent', () => {
  let component: OrganisationseinheitFormComponent;
  let fixture: ComponentFixture<OrganisationseinheitFormComponent>;
  let form: UntypedFormGroup;

  const organisationseinheitService: Mock<OrganisationseinheitService> = mock(OrganisationseinheitService);

  const saveButtonSelector: string = getDataTestIdOf('organisationseinheit-save-button');
  const closeButtonSelector: string = getDataTestIdOf('organisationseinheit-close-button');
  const headerSelector: string = getDataTestIdOf('organisationseinheit-form-header');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        OrganisationseinheitFormComponent,
        MockComponent(TextFieldComponent),
        MockComponent(ButtonWithSpinnerComponent),
      ],
      imports: [ReactiveFormsModule, FormsModule, ButtonWithSpinnerComponent],
      providers: [{ provide: OrganisationseinheitService, useValue: organisationseinheitService }],
    }).compileComponents();

    fixture = TestBed.createComponent(OrganisationseinheitFormComponent);
    component = fixture.componentInstance;
    form = fixture.componentInstance.formService.form;
    fixture.detectChanges();

    component.dialog.showModal = jest.fn();
    component.dialog.close = jest.fn();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('form element', () => {
    const fields: string[][] = [
      [OrganisationseinheitFormService.ORGANISATIONSEINHEIT_NAME_FIELD, 'Name', 'organisationseinheit-name'],
      [OrganisationseinheitFormService.ORGANISATIONSEINHEIT_IDS_FIELD, 'OrganisationseinheitID', 'organisationseinheit-id'],
    ];

    it.each(fields)('should have label for field "%s" with name "%s"', (fieldName: string, text: string, inputId: string) => {
      const textFieldElement = getElementFromFixture(fixture, getDataTestIdOf(inputId));
      expect(textFieldElement.getAttribute('label')).toBe(text);
    });

    it.each(fields)('should bind form for text-field "%s"', (fieldName, text, dataTestId) => {
      const fieldValue: string = `some text-field ${text}`;
      const formControl: AbstractControl = form.get(fieldName);

      const textFieldComponent: DebugElement = getDebugElementFromFixtureByCss(fixture, getDataTestIdOf(dataTestId));
      ngMocks.change(textFieldComponent, fieldValue);
      expect(formControl.value).toBe(fieldValue);
    });
  });

  describe('save button', () => {
    let saveButtonComponent: ButtonWithSpinnerComponent;

    beforeEach(() => {
      saveButtonComponent = getDebugElementFromFixtureByCss(fixture, saveButtonSelector).componentInstance;
    });

    it('should call submit on click', () => {
      component.submit = jest.fn();

      dispatchEventFromFixture(fixture, saveButtonSelector, 'clickEmitter');

      expect(component.submit).toHaveBeenCalled();
    });

    it('should be disabled while stateResource in loading', () => {
      component.organisationseinheitItems = createEmptyStateResource(true);

      fixture.detectChanges();

      expect(saveButtonComponent.stateResource.loading).toBe(true);
    });

    it('should be enabled while not in progress', () => {
      component.organisationseinheitItems = createEmptyStateResource(false);

      fixture.detectChanges();

      expect(saveButtonComponent.stateResource.loading).toBe(false);
    });
  });

  describe('submit', () => {
    beforeEach(() => {
      component.handleProgressChange = jest.fn();
    });

    it('should not call complete on submit error', fakeAsync(() => {
      component.formService.submit = () => throwError(() => new Error('some error'));

      component.submit();
      component.submitInProgress$.subscribe({
        error: () => {},
      });
      tick();

      expect(component.handleProgressChange).not.toHaveBeenCalled();
    }));

    it('should call complete on submit event', fakeAsync(() => {
      component.formService.submit = () => of(false);

      component.submit();
      component.submitInProgress$.subscribe();
      tick();

      expect(component.handleProgressChange).toHaveBeenCalled();
    }));

    it.each([true, false])('should use submit progress "%s"', (progress) => {
      component.organisationseinheitItems = createEmptyStateResource(progress);

      fixture.detectChanges();

      const saveButtonComponent: ButtonWithSpinnerComponent = getDebugElementFromFixtureByCss(
        fixture,
        saveButtonSelector,
      ).componentInstance;

      expect(saveButtonComponent.stateResource.loading).toBe(progress);
    });
  });

  describe('handle progress change', () => {
    it('should call complete if no errors with progress false', () => {
      component.completeIfNoErrors = jest.fn();

      component.handleProgressChange(false);

      expect(component.completeIfNoErrors).toHaveBeenCalled();
    });

    it('should call complete if no errors with progress true', () => {
      component.completeIfNoErrors = jest.fn();

      component.handleProgressChange(true);

      expect(component.completeIfNoErrors).not.toHaveBeenCalled();
    });
  });

  describe('complete if no errors', () => {
    beforeEach(() => {
      component.complete = jest.fn();
    });

    it('should call not complete with errors', () => {
      component.formService.isInvalid = jest.fn().mockReturnValue(true);

      component.completeIfNoErrors();

      expect(component.complete).not.toHaveBeenCalled();
    });

    it('should call complete without errors', () => {
      component.formService.isInvalid = jest.fn().mockReturnValue(false);

      component.completeIfNoErrors();

      expect(component.complete).toHaveBeenCalled();
    });
  });

  describe('complete', () => {
    beforeEach(() => {
      component.dialog.close = jest.fn();
      component.formService.reset = jest.fn();
    });

    it('should close dialog', () => {
      component.complete();

      expect(component.dialog.close).toHaveBeenCalled();
    });
    it('should reset form', () => {
      component.complete();

      expect(component.formService.reset).toHaveBeenCalled();
    });
  });

  describe('close button', () => {
    it('should call to close dialog', () => {
      component.dialog.close = jest.fn();

      dispatchEventFromFixture(fixture, closeButtonSelector, 'click');

      expect(component.dialog.close).toHaveBeenCalled();
    });
  });

  describe('open', () => {
    beforeEach(() => {
      form.markAsTouched();
    });

    it('should set create label', () => {
      component.open();

      expect(component.label).toEqual(OrganisationseinheitFormComponent.CREATE_LABEL);
    });

    it('should open form', () => {
      component.open();

      expect(component.dialog.showModal).toHaveBeenCalled();
    });

    it('should reset form', () => {
      component.formService.reset = jest.fn();

      component.open();

      expect(component.formService.reset).toHaveBeenCalled();
    });
  });

  describe('open edit', () => {
    const organisationseinheit: Organisationseinheit = createOrganisationseinheit();

    it('should set edit label', () => {
      component.openEdit(organisationseinheit);

      expect(component.label).toEqual(OrganisationseinheitFormComponent.EDIT_LABEL);
    });

    it('should open dialog', () => {
      component.open();

      expect(component.dialog.showModal).toHaveBeenCalled();
    });

    it('should patch form', () => {
      component.formService.patch = jest.fn();

      component.openEdit(organisationseinheit);

      expect(component.formService.patch).toHaveBeenCalledWith(organisationseinheit);
    });
  });

  describe('header', () => {
    it('should show label text', () => {
      const text: string = 'test-text';
      component.label = text;

      fixture.detectChanges();

      const headerElement: HTMLElement = getElementFromFixture(fixture, headerSelector);
      expect(headerElement.textContent.trim()).toEqual(text);
    });
  });
});
