/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import {
  BlobWithFileName,
  StateResource,
  createEmptyStateResource,
  createStateResource,
} from '@alfa-client/tech-shared';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { SnackBarService } from '@alfa-client/ui';
import { HttpErrorResponse, HttpResponse } from '@angular/common/http';
import { fakeAsync, tick } from '@angular/core/testing';
import { faker } from '@faker-js/faker';
import { Resource, ResourceUri } from '@ngxp/rest';
import { cold, hot } from 'jest-marbles';
import { createBinaryFileResource, createBlob } from 'libs/binary-file-shared/test/binary-file';
import {
  VALIDATION_MESSAGES,
  ValidationMessageCode,
} from 'libs/tech-shared/src/lib/validation/tech.validation.messages';
import { DummyLinkRel } from 'libs/tech-shared/test/dummy';
import { createDummyResource } from 'libs/tech-shared/test/resource';
import { Observable, of } from 'rxjs';
import { createHttpErrorResponse } from '../../../tech-shared/test/http';
import { singleHot } from '../../../tech-shared/test/marbles';
import { BinaryFileResource } from './binary-file.model';
import { BinaryFileRepository } from './binary-file.repository';
import { BinaryFileService } from './binary-file.service';

describe('BinaryFileService', () => {
  let service: BinaryFileService;

  let repository: Mock<BinaryFileRepository>;
  let snackBarService: Mock<SnackBarService>;

  beforeEach(() => {
    repository = mock(BinaryFileRepository);
    snackBarService = mock(SnackBarService);

    service = new BinaryFileService(useFromMock(repository), useFromMock(snackBarService));
  });

  it('should created', () => {
    expect(service).toBeTruthy();
  });

  describe('download file', () => {
    const binaryFileResource: BinaryFileResource = createBinaryFileResource();
    const downloadNamePrefix: string = 'VorgangsNummerAsPrefix';

    const blob = {};

    beforeEach(() => {
      service.save = jest.fn();
      repository.download.mockReturnValue(hot('a', { a: blob }));
    });

    it('should call repository', () => {
      service.downloadFile(binaryFileResource, downloadNamePrefix);

      expect(repository.download).toHaveBeenCalledWith(binaryFileResource);
    });

    it('should return value', () => {
      repository.download.mockReturnValue(singleHot(blob, '-a'));

      const returnValue: Observable<StateResource<Blob>> = service.downloadFile(
        binaryFileResource,
        downloadNamePrefix,
      );

      expect(returnValue).toBeObservable(
        cold('ab', {
          a: createEmptyStateResource(true),
          b: createStateResource(blob),
        }),
      );
    });

    describe('save data', () => {
      describe('on existing response', () => {
        it('should save file without prefix', () => {
          service.saveBinaryFile(blob, binaryFileResource, undefined);

          expect(service.save).toHaveBeenCalledWith(blob, binaryFileResource.name);
        });

        it('should save file with prefix', () => {
          service.saveBinaryFile(blob, binaryFileResource, downloadNamePrefix);

          expect(service.save).toHaveBeenCalledWith(
            blob,
            downloadNamePrefix + '_' + binaryFileResource.name,
          );
        });

        it('should return loaded stateResource', () => {
          const result: StateResource<Blob> = service.saveBinaryFile(
            blob,
            binaryFileResource,
            downloadNamePrefix,
          );

          expect(result).toEqual(createStateResource(blob));
        });
      });

      describe('on non existing response', () => {
        it('should return loading stateResource', () => {
          const result: StateResource<Blob> = service.saveBinaryFile(
            undefined,
            binaryFileResource,
            downloadNamePrefix,
          );

          expect(result).toEqual(createEmptyStateResource(true));
        });
      });
    });
  });

  describe('download archive', () => {
    const blob: Blob = createBlob();
    const resourceUri: ResourceUri = faker.internet.url();

    beforeEach(() => {
      service.save = jest.fn();
      repository.downloadArchive.mockReturnValue(hot('a', { a: blob }));
    });

    it('should call repository', () => {
      service.downloadArchive(resourceUri);

      expect(repository.downloadArchive).toHaveBeenCalledWith(resourceUri);
    });

    it('should return value', () => {
      repository.downloadArchive.mockReturnValue(singleHot(blob, '-a'));
      service.saveData = jest.fn().mockReturnValue(createStateResource(blob));

      const returnValue: Observable<StateResource<Blob>> = service.downloadArchive(resourceUri);

      expect(returnValue).toBeObservable(
        cold('ab', {
          a: createEmptyStateResource(true),
          b: createStateResource(blob),
        }),
      );
    });

    describe('save data', () => {
      const fileName: string = faker.random.word();
      const blobWithFileName: BlobWithFileName = { blob, fileName };

      describe('on existing response', () => {
        it('should save file', () => {
          service.saveData(blobWithFileName);

          expect(service.save).toHaveBeenCalledWith(blob, fileName);
        });

        it('should return loaded stateResource', () => {
          const result: StateResource<Blob> = service.saveData(blobWithFileName);

          expect(result).toEqual(createStateResource(blob));
        });
      });

      describe('on non existing response', () => {
        it('should return loading stateResource', () => {
          const result: StateResource<Blob> = service.saveData({
            ...blobWithFileName,
            blob: undefined,
          });

          expect(result).toEqual(createEmptyStateResource(true));
        });
      });
    });
  });

  describe('upload file', () => {
    const dummyResource: Resource = createDummyResource();
    const dummyLinkRel: string = DummyLinkRel.DUMMY;

    const fileLocation: string = 'fileLocation';
    const uploadFileResponse: HttpResponse<Object> = <any>{
      response: { headers: { ['Location']: fileLocation } },
    };

    const testFile: any = <File>{ name: 'fileName' };
    const returnFile: BinaryFileResource = createBinaryFileResource();

    beforeEach(() => {
      repository.uploadFile.mockReturnValue(hot('-a', { a: uploadFileResponse }));
      repository.getFile.mockReturnValue(of(returnFile));
    });

    it('should call repository', () => {
      service.uploadFile(dummyResource, dummyLinkRel, testFile);

      expect(repository.uploadFile).toHaveBeenCalledWith(dummyResource, dummyLinkRel, testFile);
    });

    it.skip('should call get file', fakeAsync(() => {
      repository.uploadFile.mockReturnValue(of(uploadFileResponse));
      repository.getFile.mockReturnValue(of(returnFile));

      service.uploadFile(dummyResource, dummyLinkRel, testFile).subscribe();
      tick();

      expect(repository.getFile).toHaveBeenCalledWith(fileLocation);
    }));

    describe('handleErrorByStaus', () => {
      const showValidationErrorSnackBar: boolean = true;

      beforeEach(() => {
        service.handleSnackBar = jest.fn();
      });

      it('should call handle snackBar on 422 error', () => {
        const response: HttpErrorResponse = buildUnprocessableEntityErrorResponse();

        service.handleErrorByStatus(response, showValidationErrorSnackBar);

        expect(service.handleSnackBar).toHaveBeenCalledWith(response, showValidationErrorSnackBar);
      });

      it('should not call handle snackBar on other error', () => {
        const response: HttpErrorResponse = buildUnprocessableEntityErrorResponse();

        service.handleErrorByStatus(response, showValidationErrorSnackBar);

        expect(snackBarService.showError).not.toHaveBeenCalled();
      });
    });

    describe('handleSnackBar', () => {
      it('should call snackbarService if should show', () => {
        service.handleSnackBar(buildUnprocessableEntityErrorResponse(), true);

        expect(snackBarService.showError).toHaveBeenCalledWith(
          VALIDATION_MESSAGES[ValidationMessageCode.FIELD_FILE_SIZE_EXCEEDED],
        );
      });

      it('should not call snackbarService if should not show', () => {
        service.handleSnackBar(buildUnprocessableEntityErrorResponse(), false);

        expect(snackBarService.showError).not.toHaveBeenCalled();
      });

      it('should not call snackbarService if not file size exceeded error', () => {
        service.handleSnackBar(
          buildUnprocessableEntityErrorResponse(ValidationMessageCode.FIELD_EMPTY),
          true,
        );

        expect(snackBarService.showError).not.toHaveBeenCalled();
      });
    });

    function buildUnprocessableEntityErrorResponse(
      validationMessageCode: ValidationMessageCode = ValidationMessageCode.FIELD_FILE_SIZE_EXCEEDED,
    ): HttpErrorResponse {
      return <HttpErrorResponse>{
        status: 422,
        error: {
          invalidParams: [
            {
              reason: validationMessageCode,
              constraintParameters: [],
            },
          ],
        },
      };
    }
  });

  describe('handleErrorByStatus', () => {
    let handleSnackBar: jest.Mock;
    let internalServerError: HttpErrorResponse;
    let unproccessableEntityError: HttpErrorResponse;

    beforeEach(() => {
      service.handleSnackBar = handleSnackBar = jest.fn();
      internalServerError = { ...createHttpErrorResponse(), status: 500 };
      unproccessableEntityError = { ...createHttpErrorResponse(), status: 422 };
    });

    it('should handleSnackBar', () => {
      service.handleErrorByStatus(unproccessableEntityError, true);

      expect(handleSnackBar).toHaveBeenCalledWith(unproccessableEntityError, true);
    });

    it('should not handleSnackbar', () => {
      service.handleErrorByStatus(internalServerError, true);

      expect(handleSnackBar).not.toHaveBeenCalled();
    });

    it('should create error state resource', () => {
      const stateResource = service.handleErrorByStatus(unproccessableEntityError, true);

      expect(stateResource.error).toEqual(unproccessableEntityError.error);
    });
  });

  describe('get file', () => {
    const responseResource: Resource = createDummyResource();

    const uri: string = faker.internet.url();

    beforeEach(() => {
      repository.getFile.mockReturnValue(hot('-a', { a: responseResource }));
    });

    it('should call repository', () => {
      service.getFile(uri);

      expect(repository.getFile).toHaveBeenCalledWith(uri);
    });

    it('should return value', () => {
      const fileList = service.getFile(uri);

      expect(fileList).toBeObservable(
        cold('ab', { a: createEmptyStateResource(true), b: createStateResource(responseResource) }),
      );
    });
  });
});
