import { OrganisationsEinheitResource } from '@alfa-client/collaboration-shared';
import { EventData, Mock, getMockComponent, mock, triggerEvent } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ReactiveFormsModule } from '@angular/forms';
import faker from '@faker-js/faker';
import { InstantSearchComponent } from '@ods/system';
import { createOrganisationsEinheitResource } from 'libs/collaboration-shared/test/organisations-einheit';
import { InstantSearchResult } from 'libs/design-system/src/lib/instant-search/instant-search/instant-search.model';
import { createInstantSearchResult } from 'libs/design-system/src/test/search';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { SearchOrganisationsEinheitFormService } from '../search-organisations-einheit.formservice';
import { SearchOrganisationsEinheitFormComponent } from './search-organisations-einheit-form.component';

describe('SearchOrganisationsEinheitFormComponent', () => {
  let component: SearchOrganisationsEinheitFormComponent;
  let fixture: ComponentFixture<SearchOrganisationsEinheitFormComponent>;

  const searchComp: string = getDataTestIdOf('search');

  const formService: Mock<SearchOrganisationsEinheitFormService> = mock(
    SearchOrganisationsEinheitFormService,
  );

  const instantSearchResult: InstantSearchResult<OrganisationsEinheitResource> =
    createInstantSearchResult<OrganisationsEinheitResource>();
  const organisationsEinheitResource: OrganisationsEinheitResource =
    createOrganisationsEinheitResource();

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      imports: [ReactiveFormsModule],
      declarations: [
        SearchOrganisationsEinheitFormComponent,
        MockComponent(InstantSearchComponent),
      ],
      providers: [
        {
          provide: SearchOrganisationsEinheitFormService,
          useValue: formService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(SearchOrganisationsEinheitFormComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('set organisationsEinheiten', () => {
    it('should map organisationsEinheitResources', () => {
      component.mapOrganisationsEinheitResources = jest.fn();

      component.organisationsEinheiten = [organisationsEinheitResource];

      expect(component.mapOrganisationsEinheitResources).toHaveBeenCalledWith([
        organisationsEinheitResource,
      ]);
    });
  });

  describe('map organisationsEinheit Resources', () => {
    it('should call mapToInstantSearchResult', () => {
      component.mapToInstantantSearchResult = jest.fn();

      component.mapOrganisationsEinheitResources([organisationsEinheitResource]);

      expect(component.mapToInstantantSearchResult).toHaveBeenCalledWith(
        organisationsEinheitResource,
      );
    });

    it('should set searchResults', () => {
      component.mapToInstantantSearchResult = jest.fn().mockReturnValue(instantSearchResult);

      component.mapOrganisationsEinheitResources([organisationsEinheitResource]);

      expect(component.searchResults).toEqual([instantSearchResult]);
    });
  });

  describe('map to instand search result', () => {
    it('should map titel', () => {
      const instantSearchResult: InstantSearchResult<OrganisationsEinheitResource> =
        component.mapToInstantantSearchResult(organisationsEinheitResource);

      expect(instantSearchResult.title).toBe(organisationsEinheitResource.name);
    });
    it('should map description', () => {
      const instantSearchResult: InstantSearchResult<OrganisationsEinheitResource> =
        component.mapToInstantantSearchResult(organisationsEinheitResource);

      const expectedDescription: string = `${organisationsEinheitResource.anschrift.strasse} ${organisationsEinheitResource.anschrift.hausnummer}, ${organisationsEinheitResource.anschrift.plz} ${organisationsEinheitResource.anschrift.ort}`;
      expect(instantSearchResult.description).toBe(expectedDescription);
    });

    it('should map data', () => {
      const instantSearchResult: InstantSearchResult<OrganisationsEinheitResource> =
        component.mapToInstantantSearchResult(organisationsEinheitResource);

      expect(instantSearchResult.data).toBe(organisationsEinheitResource);
    });
  });

  describe('instant search component', () => {
    it('should be called with search results', () => {
      component.searchResults = [instantSearchResult];

      fixture.detectChanges();

      expect(getInstantSearchComponent().searchResults).toEqual([instantSearchResult]);
    });

    function getInstantSearchComponent(): InstantSearchComponent {
      return getMockComponent<InstantSearchComponent>(fixture, InstantSearchComponent);
    }

    it('should emit selected search result on searchResultSelected output', () => {
      const selectSearchResultSpy: jest.SpyInstance = (component.selectSearchResult.emit =
        jest.fn());
      const eventData: EventData<SearchOrganisationsEinheitFormComponent> = {
        fixture,
        elementSelector: searchComp,
        name: 'searchResultSelected',
        data: { data: organisationsEinheitResource },
      };

      triggerEvent(eventData);

      expect(selectSearchResultSpy).toHaveBeenCalledWith(organisationsEinheitResource);
    });

    it('should emit search on searchQueryChanged output', () => {
      const searchSpy: jest.SpyInstance = (component.search.emit = jest.fn());
      const searchBy: string = faker.random.word();
      const eventData: EventData<SearchOrganisationsEinheitFormComponent> = {
        fixture,
        elementSelector: searchComp,
        name: 'searchQueryChanged',
        data: { searchBy },
      };

      triggerEvent(eventData);

      expect(searchSpy).toHaveBeenCalledWith(searchBy);
    });

    describe('should emit clear search result', () => {
      let clearSearchResultSpy: jest.SpyInstance;
      let eventData: EventData<SearchOrganisationsEinheitFormComponent>;

      beforeEach(() => {
        clearSearchResultSpy = component.clearSearchResult.emit = jest.fn();
        eventData = {
          fixture,
          elementSelector: searchComp,
          name: 'TBD',
          data: { searchBy: faker.random.word() },
        };
      });

      it('on searchResultClosed output', () => {
        eventData = { ...eventData, name: 'searchResultClosed' };

        triggerEvent(eventData);

        expect(clearSearchResultSpy).toHaveBeenCalledWith();
      });

      it('on searchQueryCleared output', () => {
        eventData = { ...eventData, name: 'searchResultClosed' };

        triggerEvent(eventData);

        expect(clearSearchResultSpy).toHaveBeenCalled();
      });
    });
  });
});
