/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

import { TestBed } from '@angular/core/testing';
import { LoadBinaryFileListProps, loadBinaryFileList } from '@alfa-client/binary-file-shared';
import { ApiError, ApiErrorAction, MessageCode } from '@alfa-client/tech-shared';
import { Mock, mock } from '@alfa-client/test-utils';
import { Messages, SnackBarService } from '@alfa-client/ui';
import { provideMockActions } from '@ngrx/effects/testing';
import { Action } from '@ngrx/store';
import { TypedAction } from '@ngrx/store/src/models';
import { provideMockStore } from '@ngrx/store/testing';
import { hot } from 'jasmine-marbles';
import { createBinaryFileListResource } from 'libs/binary-file-shared/test/binary-file';
import { createPostfachMailResource } from 'libs/postfach-shared/test/postfach';
import { createApiError, createIssue } from 'libs/tech-shared/test/error';
import { Observable, of } from 'rxjs';
import { PostfachMailLinkRel } from '../postfach.linkrel';
import { PostfachMailResource } from '../postfach.model';
import { PostfachNachrichtProps } from './postfach.actions.model';
import { PostfachEffects } from './postfach.effects';

import * as PostfachActions from './postfach.actions';

describe('PostfachEffects', () => {
  let actions: Observable<Action>;
  let effects: PostfachEffects;

  const snackBarService: Mock<SnackBarService> = mock(SnackBarService);

  beforeEach(() => {
    TestBed.configureTestingModule({
      providers: [
        PostfachEffects,
        provideMockActions(() => actions),
        provideMockStore(),
        {
          provide: SnackBarService,
          useValue: snackBarService,
        },
      ],
    });
    effects = TestBed.inject(PostfachEffects);
  });

  describe('downloadAsPdfFailed', () => {
    describe('on usermanager service unavailable message code', () => {
      it('should call snackbar service with error message', () => {
        const apiError: ApiError = {
          ...createApiError(),
          issues: [{ ...createIssue(), messageCode: MessageCode.USER_MANAGER_SERVICE_UNAVAILABLE }],
        };
        const action: ApiErrorAction & TypedAction<string> = PostfachActions.downloadAsPdfFailed({
          apiError,
        });

        actions = hot('-a', { a: action });

        effects.downloadAsPdfFailed$.subscribe(() => {
          expect(snackBarService.showError).toHaveBeenCalledWith(
            Messages.HTTP_USER_MANAGER_SERVICE_UNAVAILABLE,
          );
        });
      });
    });

    describe('on other message code', () => {
      it('should not call snackbar service', () => {
        const apiError: ApiError = {
          ...createApiError(),
          issues: [{ ...createIssue(), messageCode: 'nonMatchingMessageCode' }],
        };
        const action: ApiErrorAction & TypedAction<string> = PostfachActions.downloadAsPdfFailed({
          apiError,
        });

        actions = hot('-a', { a: action });

        effects.downloadAsPdfFailed$.subscribe(() => {
          expect(snackBarService.showError).not.toHaveBeenCalledWith();
        });
      });
    });
  });

  describe('loadAttachmentList', () => {
    const postfachNachricht: PostfachMailResource = createPostfachMailResource();
    const action: PostfachNachrichtProps & TypedAction<string> = PostfachActions.loadAttachmentList(
      { postfachNachricht },
    );
    const props: LoadBinaryFileListProps = <any>{};

    it('should dispatch "loadAttachmentList" action', () => {
      effects.createLoadAttachmentListProps = jest.fn().mockReturnValue(props);

      actions = hot('-a-|', { a: action });

      const expected = hot('-a-|', { a: loadBinaryFileList(props) });
      expect(effects.loadAttachmentList$).toBeObservable(expected);
    });

    it('should create props', () => {
      effects.createLoadAttachmentListProps = jest.fn();
      actions = of(action);

      effects.loadAttachmentList$.subscribe();

      expect(effects.createLoadAttachmentListProps).toHaveBeenCalledWith(postfachNachricht);
    });

    describe('createLoadAttachmentListProps', () => {
      it('should have resource', () => {
        const props: LoadBinaryFileListProps =
          effects.createLoadAttachmentListProps(postfachNachricht);

        expect(props.resource).toBe(postfachNachricht);
      });

      it('should have linkRel', () => {
        const props: LoadBinaryFileListProps =
          effects.createLoadAttachmentListProps(postfachNachricht);

        expect(props.linkRel).toBe(PostfachMailLinkRel.ATTACHMENTS);
      });

      it('should have success action', () => {
        const props: LoadBinaryFileListProps =
          effects.createLoadAttachmentListProps(postfachNachricht);

        expect(props.successAction(createBinaryFileListResource()).type).toEqual(
          '[Postfach] Load attachmentList success',
        );
      });

      it('should have failure action', () => {
        const props: LoadBinaryFileListProps =
          effects.createLoadAttachmentListProps(postfachNachricht);

        expect(props.failureAction(createApiError()).type).toEqual(
          '[Postfach] Load attachmentList failure',
        );
      });
    });
  });
});
