/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import {
  createPostfachMail,
  createPostfachMailListResource,
  createPostfachMailResource,
} from 'libs/postfach-shared/test/postfach';
import { toResource } from 'libs/tech-shared/test/resource';
import { PostfachMailListLinkRel } from './postfach.linkrel';
import { PostfachNachrichtMessageCode } from './postfach.message-code';
import {
  CreatePostfachMailCommand,
  Direction,
  PostfachMail,
  PostfachMailListResource,
  PostfachMailResource,
  PostfachNachrichtenCount,
  PostfachOrder,
} from './postfach.model';
import {
  createResendPostfachMailCommand,
  createSendPostfachMailCommand,
  getMessageCodeMessage,
  getPostfachNachrichtenCount,
  isIncomingMail,
} from './postfach.util';

describe('PostfachUtil', () => {
  describe('createSendPostfachMailCommand', () => {
    const postfachMail: PostfachMail = createPostfachMail();

    it('should have order', () => {
      const command: CreatePostfachMailCommand = createSendPostfachMailCommand(postfachMail);

      expect(command.order).toEqual(PostfachOrder.SEND_POSTFACH_NACHRICHT);
    });

    it('should have body', () => {
      const command: CreatePostfachMailCommand = createSendPostfachMailCommand(postfachMail);

      expect(command.body).toEqual(postfachMail);
    });
  });

  describe('createResendPostfachMailCommand', () => {
    it('should have order', () => {
      const command: CreatePostfachMailCommand = createResendPostfachMailCommand();

      expect(command.order).toEqual(PostfachOrder.RESEND_POSTFACH_MAIL);
    });

    it('should have body', () => {
      const command: CreatePostfachMailCommand = createResendPostfachMailCommand();

      expect(command.body).toEqual(null);
    });
  });

  describe('isIncomingMail', () => {
    it('should be false on direction OUT', () => {
      var result = isIncomingMail({ ...createPostfachMailResource(), direction: Direction.OUT });

      expect(result).toBeFalsy();
    });

    it('should be true on direction IN', () => {
      var result = isIncomingMail({ ...createPostfachMailResource(), direction: Direction.IN });

      expect(result).toBeTruthy();
    });
  });

  describe('getMessageCodeMessage', () => {
    it(
      'should return message for messageCode ' + PostfachNachrichtMessageCode.CONNECTION_FAILED,
      () => {
        var result = getMessageCodeMessage({
          ...createPostfachMailResource(),
          messageCode: PostfachNachrichtMessageCode.CONNECTION_FAILED,
        });

        expect(result).toEqual(
          'Verbindungsfehler: Die Verbindung zum Zielserver des Osi-Postfachs ist nicht zustandegekommen.',
        );
      },
    );

    it(
      'should return message for messageCode ' + PostfachNachrichtMessageCode.PROCESSING_FAILED,
      () => {
        var result = getMessageCodeMessage({
          ...createPostfachMailResource(),
          messageCode: PostfachNachrichtMessageCode.PROCESSING_FAILED,
        });

        expect(result).toEqual(
          'Fehlerhafte Verarbeitung: Die Nachricht wurde an das Osi-Postfach geschickt, konnte dort aber nicht richtig verarbeitet werden.',
        );
      },
    );
  });

  describe('getPostfacNachrichtenCount', () => {
    describe('return NONE', () => {
      it('on null listResource', () => {
        var count: PostfachNachrichtenCount = getPostfachNachrichtenCount(null);

        expect(count).toBe(PostfachNachrichtenCount.NONE);
      });

      it('on undefined listResource', () => {
        var count: PostfachNachrichtenCount = getPostfachNachrichtenCount(undefined);

        expect(count).toBe(PostfachNachrichtenCount.NONE);
      });

      it('should return NONE on empty resource list', () => {
        var listResource: PostfachMailListResource = createListResource([]);

        var count: PostfachNachrichtenCount = getPostfachNachrichtenCount(listResource);

        expect(count).toBe(PostfachNachrichtenCount.NONE);
      });

      it('should return NONE on non existing resources', () => {
        var listResource: PostfachMailListResource = toResource({}, [], {});

        var count: PostfachNachrichtenCount = getPostfachNachrichtenCount(listResource);

        expect(count).toBe(PostfachNachrichtenCount.NONE);
      });
    });

    it('should return SINGLE on one existing resources', () => {
      var listResource: PostfachMailListResource = createListResource([
        createPostfachMailResource(),
      ]);

      var count: PostfachNachrichtenCount = getPostfachNachrichtenCount(listResource);

      expect(count).toBe(PostfachNachrichtenCount.SINGLE);
    });

    it('should return MULTIPLE on multiple existing resources', () => {
      var listResource: PostfachMailListResource = createPostfachMailListResource();

      var count: PostfachNachrichtenCount = getPostfachNachrichtenCount(listResource);

      expect(count).toBe(PostfachNachrichtenCount.MULTIPLE);
    });

    function createListResource(
      postfachMailResources: PostfachMailResource[],
    ): PostfachMailListResource {
      return toResource({}, [], {
        [PostfachMailListLinkRel.POSTFACH_MAIL_LIST]: postfachMailResources,
      });
    }
  });
});
