/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { createStateResource, HasLinkPipe } from '@alfa-client/tech-shared';
import {
  existsAsHtmlElement,
  Mock,
  mock,
  notExistsAsHtmlElement,
  useFromMock,
} from '@alfa-client/test-utils';
import {
  NotificationSendForTypes,
  UserNotificationsTypes,
  UserSettingsLinkRel,
} from '@alfa-client/user-settings-shared';
import { EventEmitter } from '@angular/core';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { toResource } from 'libs/tech-shared/test/resource';
import { SlideToggleComponent } from 'libs/ui/src/lib/ui/slide-toggle/slide-toggle.component';
import { UserSettings } from 'libs/user-settings-shared/src/lib/user-settings.model';
import {
  createUserSettings,
  createUserSettingsResource,
} from 'libs/user-settings-shared/test/user-settings';
import { MockComponent } from 'ng-mocks';
import { UserSettingsEmailBenachrichtigungComponent } from './user-settings-email-benachrichtigung.component';

describe('UserSettingsEmailBenachrichtigungComponent', () => {
  let component: UserSettingsEmailBenachrichtigungComponent;
  let fixture: ComponentFixture<UserSettingsEmailBenachrichtigungComponent>;

  const vorgangCreatedToggle: string = getDataTestIdOf('email-benachrichtigung-neuer-Vorgang');
  const postfachNachrichtFromAntragstellerToggle: string = getDataTestIdOf(
    'email-benachrichtigung-neue-Nachricht-Antragsteller',
  );
  const vorgangAssignedToUserToggle: string = getDataTestIdOf(
    'email-benachrichtigung-neuer-Vorgang-Assigned',
  );
  const wiedervorlageDueTodayToggle: string = getDataTestIdOf(
    'email-benachrichtigung-Faellige-Wiedervorlage',
  );

  const valueChangedEmitter: Mock<EventEmitter<UserSettings>> = {
    ...mock(EventEmitter),
    emit: jest.fn(),
  };

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        HasLinkPipe,
        UserSettingsEmailBenachrichtigungComponent,
        MockComponent(SlideToggleComponent),
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(UserSettingsEmailBenachrichtigungComponent);
    component = fixture.componentInstance;
    component.userSettings = createStateResource(
      createUserSettingsResource([UserSettingsLinkRel.EDIT]),
    );
    component.valueChanged = useFromMock(valueChangedEmitter);
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('component', () => {
    describe('isChecked', () => {
      it('should return false if notificationType is not in resource', () => {
        component.resourceHasKey = jest.fn().mockReturnValue(false);

        const result: boolean = component.isChecked(UserNotificationsTypes.VORGANGE_CREATED);

        expect(result).toEqual(false);
      });

      it('should return true if notificationType is true in resource', () => {
        component.resourceHasKey = jest.fn().mockReturnValue(true);
        component.userSettings.resource[UserNotificationsTypes.VORGANGE_CREATED] = true;

        const result: boolean = component.isChecked(UserNotificationsTypes.VORGANGE_CREATED);

        expect(result).toEqual(true);
      });

      it('should return false if notificationType is false in resource', () => {
        component.resourceHasKey = jest.fn().mockReturnValue(true);
        component.userSettings.resource[UserNotificationsTypes.VORGANGE_CREATED] = false;

        const result: boolean = component.isChecked(UserNotificationsTypes.VORGANGE_CREATED);

        expect(result).toEqual(false);
      });

      it('should call return true if notificationsSendFor is ALL', () => {
        component.userSettings.resource[UserNotificationsTypes.NOTIFICATIONS_SEND_FOR] =
          NotificationSendForTypes.ALL;

        const result: boolean = component.isChecked(UserNotificationsTypes.NOTIFICATIONS_SEND_FOR);

        expect(result).toEqual(true);
      });

      it('should call return false if notificationsSendFor is NONE', () => {
        component.userSettings.resource[UserNotificationsTypes.NOTIFICATIONS_SEND_FOR] =
          NotificationSendForTypes.NONE;

        const result: boolean = component.isChecked(UserNotificationsTypes.NOTIFICATIONS_SEND_FOR);

        expect(result).toEqual(false);
      });
    });

    describe('resourceHasKey', () => {
      it('should return true if notificationType is in resource', () => {
        component.userSettings.resource[UserNotificationsTypes.VORGANGE_CREATED] = true;

        const result: boolean = component.resourceHasKey(UserNotificationsTypes.VORGANGE_CREATED);

        expect(result).toEqual(true);
      });

      it('should return true if notificationType is in resource', () => {
        delete component.userSettings.resource[UserNotificationsTypes.VORGANGE_CREATED];

        const result: boolean = component.resourceHasKey(UserNotificationsTypes.VORGANGE_CREATED);

        expect(result).toEqual(false);
      });
    });

    describe('emitSettingsForVorgangCreated', () => {
      it('should emit valueChanged with vorgangCreated and notificationsSendFor true if parameter is true', () => {
        const settings = createUserSettings(false);
        component.userSettings.resource = toResource(settings);

        component.emitSettingsForVorgangCreated(true);

        expect(valueChangedEmitter.emit).toBeCalledWith({
          ...settings,
          [UserNotificationsTypes.VORGANGE_CREATED]: true,
          [UserNotificationsTypes.NOTIFICATIONS_SEND_FOR]: NotificationSendForTypes.ALL,
        });
      });

      it('should emit valueChanged with vorgangCreated and notificationsSendFor false if parameter is false', () => {
        const settings = createUserSettings(true);
        component.userSettings.resource = toResource(settings);

        component.emitSettingsForVorgangCreated(false);

        expect(valueChangedEmitter.emit).toBeCalledWith({
          ...settings,
          [UserNotificationsTypes.VORGANGE_CREATED]: false,
          [UserNotificationsTypes.NOTIFICATIONS_SEND_FOR]: NotificationSendForTypes.NONE,
        });
      });
    });

    describe('emitSettings', () => {
      it('should emit valueChanged with vorgangCreated and notificationsSendFor true if parameter is true', () => {
        const settings = createUserSettings(false);
        component.userSettings.resource = toResource(settings);

        component.emitSettings(UserNotificationsTypes.WIEDERVORLAGE_DUE_TODAY, true);

        expect(valueChangedEmitter.emit).toBeCalledWith({
          ...settings,
          [UserNotificationsTypes.WIEDERVORLAGE_DUE_TODAY]: true,
        });
      });

      it('should emit valueChanged with vorgangCreated and notificationsSendFor false if parameter is false', () => {
        const settings = createUserSettings(true);
        component.userSettings.resource = toResource(settings);

        component.emitSettings(UserNotificationsTypes.WIEDERVORLAGE_DUE_TODAY, false);

        expect(valueChangedEmitter.emit).toBeCalledWith({
          ...settings,
          [UserNotificationsTypes.WIEDERVORLAGE_DUE_TODAY]: false,
        });
      });
    });
  });

  describe('template', () => {
    it('should not be empty if edit link in userSettings', () => {
      existsAsHtmlElement(fixture, vorgangCreatedToggle);
    });

    it('should be empty if edit link not in userSettings', () => {
      delete component.userSettings.resource._links[UserSettingsLinkRel.EDIT];

      fixture.detectChanges();

      existsAsHtmlElement(fixture, vorgangCreatedToggle);
    });

    describe('notification toggle', () => {
      describe('mit Label "Neuer Vorgang im Alfa"', () => {
        it('exists as HtmlElement', () => {
          existsAsHtmlElement(fixture, vorgangCreatedToggle);
        });
      });

      describe('mit Label "Neue Nachricht vom Antragsteller"', () => {
        it('exists as HtmlElement if corresponding key is in userSettings', () => {
          existsAsHtmlElement(fixture, postfachNachrichtFromAntragstellerToggle);
        });

        it('not exists as HtmlElement if corresponding key is not in userSettings', () => {
          delete component.userSettings.resource.postfachNachrichtFromAntragsteller;

          fixture.detectChanges();

          notExistsAsHtmlElement(fixture, postfachNachrichtFromAntragstellerToggle);
        });
      });

      describe('mit Label "Vorgang wurde mir zugewiesen"', () => {
        it('exists as HtmlElement if corresponding key is in userSettings', () => {
          existsAsHtmlElement(fixture, vorgangAssignedToUserToggle);
        });

        it('not exists as HtmlElement if postfachNachrichtFromAntragsteller not in userSettings', () => {
          delete component.userSettings.resource.vorgangAssignedToUser;

          fixture.detectChanges();

          notExistsAsHtmlElement(fixture, vorgangAssignedToUserToggle);
        });
      });

      describe('mit Label "Fällige Wiedervorlage"', () => {
        it('exists as HtmlElement if corresponding key is in userSettings', () => {
          existsAsHtmlElement(fixture, wiedervorlageDueTodayToggle);
        });

        it('not exists as HtmlElement if corresponding key is not in userSettings', () => {
          delete component.userSettings.resource.wiedervorlageDueToday;

          fixture.detectChanges();

          notExistsAsHtmlElement(fixture, wiedervorlageDueTodayToggle);
        });
      });
    });
  });
});
