/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.wiedervorlage;

import java.time.LocalDate;
import java.time.ZonedDateTime;
import java.util.Comparator;
import java.util.Objects;
import java.util.Optional;
import java.util.function.Predicate;
import java.util.stream.Stream;

import jakarta.validation.Valid;

import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItemService;
import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.user.CurrentUserService;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Validated
@Service
class WiedervorlageService {

	private static final Predicate<Wiedervorlage> IS_NOT_DONE = wiedervorlage -> !wiedervorlage.isDone();

	private final WiedervorlageRemoteService remoteService;
	private final VorgangAttachedItemService vorgangAttachedItemService;
	private final CurrentUserService currentUserService;

	public Command createWiedervorlage(@Valid Wiedervorlage wiedervorlage, String vorgangId) {
		return vorgangAttachedItemService.createNewWiedervorlage(addCreated(wiedervorlage), vorgangId);
	}

	Wiedervorlage addCreated(Wiedervorlage wiedervorlage) {
		return wiedervorlage.toBuilder()
				.createdAt(ZonedDateTime.now().withNano(0))
				.createdBy(currentUserService.getUserId().toString())
				.build();
	}

	public Command editWiedervorlage(@Valid Wiedervorlage wiedervorlage, String wiedervorlageId, long version) {
		return vorgangAttachedItemService.editWiedervorlage(wiedervorlage, wiedervorlageId, version);
	}

	public Wiedervorlage getById(String wiedervorlageId) {
		return remoteService.getById(wiedervorlageId);
	}

	@Async
	public void updateNextFrist(String vorgangId, Wiedervorlage changedOrNewWiedervorlage) {
		var persistedWiedervorlagen = findByVorgangId(vorgangId);
		var persistedWiedervorlagenExcludingChanged = persistedWiedervorlagen.filter(
				wiedervorlage -> !wiedervorlage.getId().equals(changedOrNewWiedervorlage.getId()));

		var wiedervorlagen = Stream.concat(persistedWiedervorlagenExcludingChanged, Stream.of(changedOrNewWiedervorlage));
		remoteService.updateNextFrist(vorgangId, calculateNextFrist(wiedervorlagen));
	}

	Optional<LocalDate> calculateNextFrist(Stream<Wiedervorlage> wiedervorlagen) {
		return wiedervorlagen
				.filter(Objects::nonNull)
				.filter(IS_NOT_DONE)
				.sorted(Comparator.comparing(Wiedervorlage::getFrist))
				.map(Wiedervorlage::getFrist)
				.findFirst();
	}

	public Stream<Wiedervorlage> findByVorgangId(String vorgangId) {
		return remoteService.findByVorgangId(vorgangId);
	}

	public Command erledigen(Wiedervorlage wiedervorlage) {
		return vorgangAttachedItemService.setWiedervorlageDone(wiedervorlage, true);
	}

	public Command wiedereroeffnen(Wiedervorlage wiedervorlage) {
		return vorgangAttachedItemService.setWiedervorlageDone(wiedervorlage, false);
	}

}