package de.ozgcloud.alfa.attachment;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.io.OutputStream;
import java.util.List;
import java.util.UUID;
import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.alfa.attachment.AttachmentController.AttachmentsByVorgangController;
import de.ozgcloud.alfa.common.binaryfile.BinaryFileModelAssembler;
import de.ozgcloud.alfa.common.file.OzgFileService;
import de.ozgcloud.alfa.common.file.OzgFileTestFactory;
import de.ozgcloud.alfa.common.zipdownload.ZipDownloadService;
import de.ozgcloud.alfa.vorgang.EingangTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangController;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import de.ozgcloud.alfa.vorgang.VorgangWithEingangTestFactory;
import lombok.SneakyThrows;

class AttachmentsByVorgangControllerTest {

	@Spy
	@InjectMocks
	private AttachmentsByVorgangController controller;
	@Mock
	private BinaryFileModelAssembler modelAssembler;
	@Mock
	private OzgFileService fileService;
	@Mock
	private ZipDownloadService zipDownloadService;
	@Mock
	private VorgangController vorgangController;

	private MockMvc mockMvc;

	@BeforeEach
	void initTest() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).build();
	}

	@DisplayName("Download")
	@Nested
	class TestDownload {
		private final String filename = LoremIpsum.getInstance().getWords(1);

		@BeforeEach
		void setUpMock() {
			doReturn(filename).when(controller).buildZipName(VorgangHeaderTestFactory.ID);
		}

		@SneakyThrows
		@Test
		void shouldRespondWithStatusOk() {
			var response = doRequest();

			response.andExpect(status().isOk());
		}

		@SneakyThrows
		@Test
		void shouldBuildZipName() {
			doRequest();

			verify(controller).buildZipName(VorgangHeaderTestFactory.ID);
		}

		@SneakyThrows
		@Test
		void shouldHaveContentDispositonHeader() {
			var response = doRequest();

			response.andExpect(header().string(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=" + filename));
		}

		@SneakyThrows
		@Test
		void shouldHaveContentType() {
			var response = doRequest();

			response.andExpect(header().string(HttpHeaders.CONTENT_TYPE, MediaType.APPLICATION_OCTET_STREAM_VALUE));
		}

		@SneakyThrows
		@Test
		void shouldWriteZipFile() {
			doRequest();

			verify(controller).createZipFile(any(), eq(EingangTestFactory.ID));
		}

		@SneakyThrows
		private ResultActions doRequest() {
			return mockMvc.perform(asyncDispatch(
					mockMvc.perform(get(AttachmentsByVorgangController.PATH.replace("{vorgangId}", VorgangHeaderTestFactory.ID) + "?eingangId="
							+ EingangTestFactory.ID)
									.header(HttpHeaders.ACCEPT, MediaType.APPLICATION_OCTET_STREAM_VALUE))
							.andReturn()))
					.andExpect(status().isOk());
		}
	}

	@Nested
	class TestBuildZipName {
		private final String vorgangId = UUID.randomUUID().toString();

		private final VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();

		@BeforeEach
		void mockVorgangController() {
			when(vorgangController.getVorgang(vorgangId)).thenReturn(vorgang);
		}

		@Test
		void shouldCallVorgangController() {
			callController();

			verify(vorgangController).getVorgang(vorgangId);
		}

		@Test
		void shouldReturnFileName() {
			var resultFileName = callController();

			assertThat(resultFileName).isEqualTo(vorgang.getNummer() + "_Anhaenge.zip");
		}

		private String callController() {
			return controller.buildZipName(vorgangId);
		}
	}

	@Nested
	class TestCreateZipFile {
		@Mock
		private OutputStream outputStream;

		@Test
		void shouldGetAttachments() {
			callController();

			verify(fileService).getAttachmentsByEingang(EingangTestFactory.ID);
		}

		@Test
		void shouldCallZipDownloadService() {
			var ozgFile = OzgFileTestFactory.create();
			when(fileService.getAttachmentsByEingang(EingangTestFactory.ID)).thenReturn(Stream.of(ozgFile));

			callController();

			verify(zipDownloadService).write(List.of(ozgFile), outputStream);
		}

		private void callController() {
			controller.createZipFile(outputStream, EingangTestFactory.ID);
		}
	}
}
