package de.ozgcloud.alfa.bescheid;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Spy;

class BescheidByBeschiedenAmComparatorTest {

	@Spy
	private BescheidByBeschiedenAmComparator comparator;

	@Nested
	class TestCompare {
		int expectedResult = -1;
		private final Bescheid olderBescheid = BescheidTestFactory.create();
		private final Bescheid recentBescheid = BescheidTestFactory.create();

		@Test
		void shouldCompareByStatus() {
			compare();

			verify(comparator).compareByStatus(olderBescheid, recentBescheid);
		}

		@Nested
		class TestWithDraft {

			@BeforeEach
			void setUpMock() {
				doReturn(expectedResult).when(comparator).compareByStatus(olderBescheid, recentBescheid);
			}

			@Test
			void shouldNotCompareByBeschiedenAm() {
				compare();

				verify(comparator, never()).compareByBeschiedenAm(any(), any());
			}

			@Test
			void shouldNotCompareBySentAt() {
				compare();

				verify(comparator, never()).compareBySentAt(any(), any());
			}

			@Test
			void shouldReturnCompareByStatus() {
				var compareResult = compare();

				assertThat(compareResult).isEqualTo(expectedResult);
			}

		}

		@Nested
		class TestWithoutDraft {

			@BeforeEach
			void mockCompareByStatus() {
				doReturn(0).when(comparator).compareByStatus(olderBescheid, recentBescheid);
			}

			@Nested
			class TestWithDifferentBeschiedenAm {

				@BeforeEach
				void setUpMock() {
					doReturn(expectedResult).when(comparator).compareByBeschiedenAm(olderBescheid, recentBescheid);
				}

				@Test
				void shouldCompareByBeschiedenAm() {
					compare();

					verify(comparator).compareByBeschiedenAm(olderBescheid, recentBescheid);
				}

				@Test
				void shouldNotCompareBySentAt() {
					compare();

					verify(comparator, never()).compareBySentAt(any(), any());
				}

				@Test
				void shouldReturnCompareByBeschiedenAmResult() {
					var compareResult = compare();

					assertThat(compareResult).isEqualTo(expectedResult);
				}
			}

			@Nested
			class TestWithSameBeschiedenAm {
				@BeforeEach
				void setUpMock() {
					doReturn(0).when(comparator).compareByBeschiedenAm(olderBescheid, recentBescheid);
				}

				@Test
				void shouldCompareByBeschiedenAm() {
					compare();

					verify(comparator).compareByBeschiedenAm(olderBescheid, recentBescheid);
				}

				@Test
				void shouldCompareBySentAt() {
					compare();

					verify(comparator).compareBySentAt(any(), any());
				}

				@Test
				void shouldReturnCompareBySentAtResult() {
					doReturn(expectedResult).when(comparator).compareBySentAt(olderBescheid, recentBescheid);

					var compareResult = compare();

					assertThat(compareResult).isEqualTo(expectedResult);
				}
			}
		}

		private int compare() {
			return comparator.compare(olderBescheid, recentBescheid);
		}
	}

	@Nested
	class TestCompareByStatus {
		private final Bescheid sentBescheid = BescheidTestFactory.createBuilder().status(BescheidStatus.SENT).build();
		private final Bescheid draftBescheid = BescheidTestFactory.createBuilder().status(BescheidStatus.DRAFT).build();

		@Test
		void shouldReturnNegative() {
			var result = comparator.compareByStatus(sentBescheid, draftBescheid);

			assertThat(result).isNegative();
		}

		@Test
		void shouldReturnPositive() {
			var result = comparator.compareByStatus(draftBescheid, sentBescheid);

			assertThat(result).isPositive();
		}

		@Test
		void shouldReturnZero() {
			var secondSentBescheid = BescheidTestFactory.createBuilder().status(BescheidStatus.SENT).build();
			var result = comparator.compareByStatus(sentBescheid, secondSentBescheid);

			assertThat(result).isZero();
		}
	}

	@Nested
	class TestCompareByBeschiedenAm {
		private final Bescheid olderBescheid = BescheidTestFactory.createBuilder().beschiedenAm("2024-05-26").build();
		private final Bescheid recentBescheid = BescheidTestFactory.createBuilder().beschiedenAm("2024-05-27").build();

		@Test
		void shouldReturnNegative() {
			var result = comparator.compareByBeschiedenAm(olderBescheid, recentBescheid);

			assertThat(result).isNegative();
		}

		@Test
		void shouldReturnPositive() {
			var result = comparator.compareByBeschiedenAm(recentBescheid, olderBescheid);

			assertThat(result).isPositive();
		}

		@Test
		void shouldReturnZero() {
			var result = comparator.compareByBeschiedenAm(olderBescheid, olderBescheid);

			assertThat(result).isZero();
		}
	}

	@Nested
	class TestComapreBySentAt {

		private final Bescheid olderBescheid = BescheidTestFactory.createBuilder()
				.sentInfo(SentInfoTestFactory.create())
				.build();
		private final Bescheid recentBescheid = BescheidTestFactory.createBuilder()
				.sentInfo(SentInfoTestFactory.createBuilder().sentAt(SentInfoTestFactory.SENT_AT.plusHours(1)).build())
				.build();

		@Test
		void shouldReturnNegative() {
			var result = comparator.compareBySentAt(olderBescheid, recentBescheid);

			assertThat(result).isNegative();
		}

		@Test
		void shouldReturnPositive() {
			var result = comparator.compareBySentAt(recentBescheid, olderBescheid);

			assertThat(result).isPositive();
		}

		@Test
		void shouldReturnZero() {
			var result = comparator.compareBySentAt(olderBescheid, olderBescheid);

			assertThat(result).isZero();
		}
	}
}
