package de.ozgcloud.alfa.bescheid;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Optional;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.bescheid.BescheidServiceGrpc.BescheidServiceBlockingStub;
import de.ozgcloud.bescheid.GrpcBescheidManagerConfigRequest;
import de.ozgcloud.bescheid.GrpcBescheidManagerConfigResponse;
import de.ozgcloud.bescheid.GrpcGetAllBescheidRequest;
import de.ozgcloud.bescheid.GrpcGetAllBescheidResponse;
import de.ozgcloud.bescheid.GrpcGetBescheidDraftRequest;
import de.ozgcloud.bescheid.GrpcGetBescheidDraftResponse;

class BescheidRemoteServiceTest {

	@Spy
	@InjectMocks
	private BescheidRemoteService service;
	@Mock
	private BescheidServiceBlockingStub bescheidServiceStub;
	@Mock
	private BescheidMapper bescheidMapper;

	@Nested
	class TestGetBescheidDraft {

		private final GrpcGetBescheidDraftRequest request = GrpcGetBescheidDraftRequestTestFactory.create();
		private final GrpcGetBescheidDraftResponse response = GrpcGetBescheidDraftResponseTestFactory.create();
		private final Bescheid bescheidDraft = BescheidTestFactory.create();

		@BeforeEach
		void setUp() {
			doReturn(request).when(service).buildGetBescheidDraftRequest(VorgangHeaderTestFactory.ID);
			when(bescheidServiceStub.getBescheidDraft(request)).thenReturn(response);
		}

		@Test
		void shouldBuildGrpcGetBescheidRequest() {
			service.getBescheidDraft(VorgangHeaderTestFactory.ID);

			verify(service).buildGetBescheidDraftRequest(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldCallGrpcService() {
			service.getBescheidDraft(VorgangHeaderTestFactory.ID);

			verify(bescheidServiceStub).getBescheidDraft(request);
		}

		@Test
		void shouldCallMapper() {
			doReturn(Optional.of(GrpcGetBescheidDraftResponseTestFactory.GRPC_BESCHEID)).when(service).getBescheidFromResponse(response);

			service.getBescheidDraft(VorgangHeaderTestFactory.ID);

			verify(bescheidMapper).fromGrpc(GrpcGetBescheidDraftResponseTestFactory.GRPC_BESCHEID, VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldReturnBescheid() {
			when(bescheidMapper.fromGrpc(GrpcGetBescheidDraftResponseTestFactory.GRPC_BESCHEID, VorgangHeaderTestFactory.ID)).thenReturn(
					bescheidDraft);

			var bescheidDraft = service.getBescheidDraft(VorgangHeaderTestFactory.ID);

			assertThat(bescheidDraft).hasValue(this.bescheidDraft);
		}

		@Test
		void shouldReturnEmpty() {
			doReturn(Optional.empty()).when(service).getBescheidFromResponse(response);

			var bescheidDraft = service.getBescheidDraft(VorgangHeaderTestFactory.ID);

			assertThat(bescheidDraft).isEmpty();
		}

		@Test
		void shouldGetBescheidFromResponse() {
			service.getBescheidDraft(VorgangHeaderTestFactory.ID);

			verify(service).getBescheidFromResponse(response);
		}
	}

	@Nested
	class TestBuildGetBescheidDraftRequest {

		@Test
		void shouldHaveVorgangId() {
			var request = service.buildGetBescheidDraftRequest(VorgangHeaderTestFactory.ID);

			assertThat(request.getVorgangId()).isEqualTo(VorgangHeaderTestFactory.ID);
		}
	}

	@Nested
	class TestGetBescheidFromResponse {

		@Test
		void shouldReturnEmpty() {
			var bescheid = service.getBescheidFromResponse(GrpcGetBescheidDraftResponse.getDefaultInstance());

			assertThat(bescheid).isEmpty();
		}

		@Test
		void shouldReturnBescheid() {
			var bescheid = service.getBescheidFromResponse(GrpcGetBescheidDraftResponseTestFactory.create());

			assertThat(bescheid).hasValue(GrpcGetBescheidDraftResponseTestFactory.GRPC_BESCHEID);
		}
	}

	@Nested
	class TestCanCreateBescheidDocument {

		private final GrpcBescheidManagerConfigRequest request = GrpcBescheidManagerConfigRequestTestFactory.create();
		private final GrpcBescheidManagerConfigResponse respone = GrpcBescheidManagerConfigResponseTestFactory.create();

		@BeforeEach
		void setUp() {
			when(bescheidServiceStub.getConfig(request)).thenReturn(respone);
		}

		@Test
		void shouldCallRemoteService() {
			service.canCreateBescheidDocument();

			verify(bescheidServiceStub).getConfig(request);
		}

		@Test
		void shouldReturnTrue() {
			var canCreate = service.canCreateBescheidDocument();

			assertThat(canCreate).isTrue();
		}

		@Test
		void shouldReturnFalseIfNoFeaturesAvailable() {
			when(bescheidServiceStub.getConfig(request)).thenReturn(GrpcBescheidManagerConfigResponse.newBuilder().build());

			var canCreate = service.canCreateBescheidDocument();

			assertThat(canCreate).isFalse();
		}

		@Test
		void shouldReturnFalseIfFeatureDisabled() {
			var respones = GrpcBescheidManagerConfigResponseTestFactory.createBuilder()
					.setFeatures(GrpcBescheidManagerFeaturesTestFactory.createBuilder()
							.setCanCreateBescheidDocument(false)
							.build())
					.build();
			when(bescheidServiceStub.getConfig(request)).thenReturn(respones);

			var canCreate = service.canCreateBescheidDocument();

			assertThat(canCreate).isFalse();
		}
	}

	@DisplayName("Find by vorgang id")
	@Nested
	class TestFindByVorgangId {

		private final GrpcGetAllBescheidRequest request = GrpcGetAllBescheidRequestTestFactory.create();
		private final GrpcGetAllBescheidResponse response = GrpcGetAllBescheidResponseTestFactory.create();

		private final Bescheid bescheid = BescheidTestFactory.create();

		@BeforeEach
		void setUp() {
			doReturn(request).when(service).buildGetAllBescheidRequest(VorgangHeaderTestFactory.ID);
			when(bescheidServiceStub.getAll(any())).thenReturn(response);

		}

		@Test
		void shouldBuildGetAllRequest() {
			service.findByVorgangId(VorgangHeaderTestFactory.ID);

			verify(service).buildGetAllBescheidRequest(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldCallRemoteService() {
			service.findByVorgangId(VorgangHeaderTestFactory.ID);

			verify(bescheidServiceStub).getAll(request);
		}

		@Test
		void shouldMapBescheid() {
			service.findByVorgangId(VorgangHeaderTestFactory.ID).toList();

			verify(bescheidMapper).fromGrpc(GrpcGetAllBescheidResponseTestFactory.GRPC_BESCHEID, VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldReturnStreamOfBescheide() {
			when(bescheidMapper.fromGrpc(any(), any())).thenReturn(bescheid);

			var bescheide = service.findByVorgangId(VorgangHeaderTestFactory.ID);

			assertThat(bescheide).containsExactly(bescheid);
		}
	}

	@DisplayName("Build get all bescheid request")
	@Nested
	class TestBuildGetAllBescheidRequest {

		@Test
		void shouldContainVorgangId() {
			var request = service.buildGetAllBescheidRequest(VorgangHeaderTestFactory.ID);

			assertThat(request.getVorgangId()).isEqualTo(VorgangHeaderTestFactory.ID);
		}
	}
}