/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.wiedervorlage;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.time.LocalDate;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import de.ozgcloud.alfa.common.binaryfile.FileId;
import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.common.command.CommandTestFactory;
import de.ozgcloud.alfa.common.command.CreateCommand;
import de.ozgcloud.alfa.common.command.LegacyOrder;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.common.errorhandling.TechnicalException;
import lombok.SneakyThrows;

class WiedervorlageCommandControllerTest {

	@Spy
	@InjectMocks
	private WiedervorlageCommandController controller;
	@Mock
	private WiedervorlageService service;

	private MockMvc mockMvc;

	@BeforeEach
	void init() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).build();
	}

	@DisplayName("Update wiedervorlage")
	@Nested
	class TestUpdateWiedervorlage {

		private static final String RESPONSE_HEADER = "http://localhost/api/commands/" + CommandTestFactory.ID;

		@Nested
		class ControllerMethods {

			@Captor
			private ArgumentCaptor<Wiedervorlage> wiedervorlageCaptor;

			@BeforeEach
			void init() {
				when(service.getById(any())).thenReturn(WiedervorlageTestFactory.create());
				when(service.editWiedervorlage(any(), any(), anyLong())).thenReturn(CommandTestFactory.createBuilder()
						.order(CommandOrder.UPDATE_ATTACHED_ITEM.name())
						.body(WiedervorlageTestFactory.createAsMap()).build());
			}

			@SneakyThrows
			@Test
			void shouldCallServiceGetById() {
				doRequest();

				verify(service).getById(WiedervorlageTestFactory.ID);
			}

			@Test
			void shouldCallServiceUpdateNextFrist() {
				doRequest();

				verify(service).updateNextFrist(eq(VorgangHeaderTestFactory.ID), wiedervorlageCaptor.capture());
				assertThat(wiedervorlageCaptor.getValue())
						.usingRecursiveComparison()
						.comparingOnlyFields("betreff", "beschreibung", "frist", "attachments")
						.isEqualTo(WiedervorlageTestFactory.create());
			}

			@SneakyThrows
			@Test
			void shouldReturnCreatedStatus() {
				var response = doRequest();

				response.andExpect(status().isCreated());
			}

			@SneakyThrows
			@Test
			void shouldResponseWithLocationHeader() {
				var response = doRequest();

				response.andExpect(header().string("Location", RESPONSE_HEADER));
			}

			@Test
			void shouldCreateCommand() {
				doRequest();

				verify(controller).createCommand(any(Wiedervorlage.class), any(CreateCommand.class));
			}

			@SneakyThrows
			private ResultActions doRequest() {
				return mockMvc.perform(
								post(WiedervorlageCommandController.WIEDERVORLAGE_COMMANDS, WiedervorlageTestFactory.ID, WiedervorlageTestFactory.VERSION)
										.content(createRequestContent()).contentType(MediaType.APPLICATION_JSON))
						.andExpect(status().is2xxSuccessful());
			}

			private String createRequestContent() {
				return WiedervorlageTestFactory
						.createRequestContent(
								CommandTestFactory.createCreateCommandBuilder()
										.order("EDIT_WIEDERVORLAGE")
										.body(WiedervorlageTestFactory.create())
										.build());
			}
		}

		@DisplayName("create command")
		@Nested
		class TestCreateCommand {

			@Captor
			private ArgumentCaptor<Wiedervorlage> wiedervorlageArgumentCaptor;

			@DisplayName("for order 'erledigen'")
			@Nested
			class TestOnErledigenOrder {

				@Test
				void shouldCallService() {
					callCreateCommand(LegacyOrder.WIEDERVORLAGE_ERLEDIGEN);

					verify(service).erledigen(any(Wiedervorlage.class));
				}

				@Test
				void shouldUpdateNextFrist() {
					callCreateCommand(LegacyOrder.WIEDERVORLAGE_ERLEDIGEN);

					verify(service).updateNextFrist(eq(VorgangHeaderTestFactory.ID), any(Wiedervorlage.class));
				}

				@Test
				void shouldSetWiedervorlageAsDone() {
					callCreateCommand(LegacyOrder.WIEDERVORLAGE_ERLEDIGEN);

					verify(service).updateNextFrist(eq(VorgangHeaderTestFactory.ID), wiedervorlageArgumentCaptor.capture());
					assertThat(wiedervorlageArgumentCaptor.getValue())
							.usingRecursiveComparison()
							.isEqualTo(WiedervorlageTestFactory.createBuilder().done(true).build());
				}
			}

			@DisplayName("for order 'wiedereroeffnen'")
			@Nested
			class TestOnWiedereroeffnenOrder {

				@Test
				void shouldCallService() {
					callCreateCommand(LegacyOrder.WIEDERVORLAGE_WIEDEREROEFFNEN);

					verify(service).wiedereroeffnen(any(Wiedervorlage.class));
				}

				@Test
				void shouldUpdateNextFrist() {
					callCreateCommand(LegacyOrder.WIEDERVORLAGE_WIEDEREROEFFNEN);

					verify(service).updateNextFrist(eq(VorgangHeaderTestFactory.ID), any(Wiedervorlage.class));
				}

				@Test
				void shouldSetWiedervorlageAsOpen() {
					callCreateCommand(LegacyOrder.WIEDERVORLAGE_WIEDEREROEFFNEN);

					verify(service).updateNextFrist(eq(VorgangHeaderTestFactory.ID), wiedervorlageArgumentCaptor.capture());
					assertThat(wiedervorlageArgumentCaptor.getValue())
							.usingRecursiveComparison()
							.isEqualTo(WiedervorlageTestFactory.createBuilder().done(false).build());
				}
			}

			@DisplayName("for order 'edit'")
			@Nested
			class TestOnEditOrder {

				@Test
				void shouldCallService() {
					callCreateCommand(LegacyOrder.EDIT_WIEDERVORLAGE);

					verify(service).editWiedervorlage(any(Wiedervorlage.class), eq(WiedervorlageTestFactory.ID),
							eq(WiedervorlageTestFactory.VERSION));
				}

				@Test
				void shouldUpdateNextFrist() {
					var wiedervorlage = WiedervorlageTestFactory.create();
					doReturn(wiedervorlage).when(controller).updateWiedervorlageByCommand(any(), any());

					callCreateCommand(LegacyOrder.EDIT_WIEDERVORLAGE);

					verify(service).updateNextFrist(VorgangHeaderTestFactory.ID, wiedervorlage);
				}

				@DisplayName("update wiedervorlage by given command")
				@Nested
				class TestUpdateWiedervorlageByCommand {

					private final static FileId ATTACHMENT_ID = FileId.from("73");
					private final static String BETREFF = "Neuer Betreff";
					private final static String BESCHREIBUNG = "Neuer Beschreibung";
					private final static LocalDate FRIST = LocalDate.now();

					@Test
					void shouldHaveSetAttachments() {
						var updatedWiedervorlage = update();

						assertThat(updatedWiedervorlage.getAttachments()).hasSize(1);
						assertThat(updatedWiedervorlage.getAttachments().get(0)).isEqualTo(ATTACHMENT_ID);
					}

					@Test
					void shouldHaveSetBetreff() {
						var updatedWiedervorlage = update();

						assertThat(updatedWiedervorlage.getBetreff()).isEqualTo(BETREFF);
					}

					@Test
					void shouldHaveSetBeschreibung() {
						var updatedWiedervorlage = update();

						assertThat(updatedWiedervorlage.getBeschreibung()).isEqualTo(BESCHREIBUNG);
					}

					@Test
					void shouldHaveSetFrist() {
						var updatedWiedervorlage = update();

						assertThat(updatedWiedervorlage.getFrist()).isEqualTo(FRIST);
					}

					private Wiedervorlage update() {
						return controller.updateWiedervorlageByCommand(WiedervorlageTestFactory.create(), buildWiedervorlage());
					}

					private Wiedervorlage buildWiedervorlage() {
						return WiedervorlageTestFactory.createBuilder()
								.clearAttachments().attachment(ATTACHMENT_ID)
								.betreff(BETREFF)
								.beschreibung(BESCHREIBUNG)
								.frist(FRIST)
								.build();
					}
				}
			}

			@Nested
			class OnUnsupportedOrder {

				@Test
				void shouldThrowException() {
					var wiedervorlage = WiedervorlageTestFactory.create();
					var command = CommandTestFactory.createCreateCommandBuilder()
							.order(CommandOrder.CREATE_ATTACHED_ITEM.name())
							.body(wiedervorlage).build();

					assertThatExceptionOfType(TechnicalException.class)
							.isThrownBy(() -> controller.createCommand(wiedervorlage, command));
				}
			}

			private Command callCreateCommand(String order) {
				return controller.createCommand(WiedervorlageTestFactory.create(),
						CommandTestFactory.createCreateCommandBuilder().body(WiedervorlageTestFactory.create())
								.order(order)
								.build());
			}
		}
	}
}