/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.wiedervorlage;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.HashMap;
import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItemTestFactory;
import de.ozgcloud.alfa.common.binaryfile.BinaryFileTestFactory;
import de.ozgcloud.alfa.common.binaryfile.FileIdMapper;
import de.ozgcloud.alfa.vorgang.GrpcVorgangAttachedItemTestFactory;
import de.ozgcloud.vorgang.common.grpc.GrpcObjectMapper;

class WiedervorlageMapperTest {

	@Spy
	@InjectMocks
	private WiedervorlageMapper mapper = Mappers.getMapper(WiedervorlageMapper.class);
	@Spy
	private FileIdMapper fileIdMapper = Mappers.getMapper(FileIdMapper.class);
	@Mock
	private GrpcObjectMapper grpcObjectMapper;

	@DisplayName("Map from item")
	@Nested
	class TestFromItem {

		@BeforeEach
		void mockMapper() {
			when(grpcObjectMapper.mapFromGrpc(any())).thenReturn(WiedervorlageTestFactory.createAsMap());
		}

		@Test
		void shouldCallGrpcObjectMapper() {
			mapper.fromItem(GrpcVorgangAttachedItemTestFactory.create());

			verify(grpcObjectMapper).mapFromGrpc(GrpcVorgangAttachedItemTestFactory.ITEM);
		}

		@Test
		void shouldCallMapperFromItemMap() {
			mapper.fromItem(GrpcVorgangAttachedItemTestFactory.create());

			verify(mapper).fromItemMap(WiedervorlageTestFactory.createAsMap(), VorgangAttachedItemTestFactory.VERSION);
		}
	}

	@DisplayName("Map from item map")
	@Nested
	class TestFromItemMap {

		@Test
		void shouldMapId() {
			var wiedervorlage = map();

			assertThat(wiedervorlage.getId()).isEqualTo(WiedervorlageTestFactory.ID);
		}

		@Test
		void shouldMapVersion() {
			var wiedervorlage = map();

			assertThat(wiedervorlage.getVersion()).isEqualTo(WiedervorlageTestFactory.VERSION);
		}

		@Test
		void shouldMapDone() {
			var wiedervorlage = map();

			assertThat(wiedervorlage.isDone()).isEqualTo(WiedervorlageTestFactory.DONE);
		}

		@Test
		void shouldMapCreatedBy() {
			var wiedervorlage = map();

			assertThat(wiedervorlage.getCreatedBy()).isEqualTo(WiedervorlageTestFactory.CREATED_BY);
		}

		@Test
		void shouldMapCreatedAt() {
			var wiedervorlage = map();

			assertThat(wiedervorlage.getCreatedAt()).isEqualTo(WiedervorlageTestFactory.CREATED_AT);
		}

		@Test
		void shouldMapBetreff() {
			var wiedervorlage = map();

			assertThat(wiedervorlage.getBetreff()).isEqualTo(WiedervorlageTestFactory.BETREFF);
		}

		@Test
		void shouldMapBeschreibung() {
			var wiedervorlage = map();

			assertThat(wiedervorlage.getBeschreibung()).isEqualTo(WiedervorlageTestFactory.BESCHREIBUNG);
		}

		@Test
		void shouldMapFrist() {
			var wiedervorlage = map();

			assertThat(wiedervorlage.getFrist()).isEqualTo(WiedervorlageTestFactory.FRIST);
		}

		@Nested
		class Attachments {

			@Test
			void shouldReturnEmptyListOnNull() {
				var wiedervorlageWithoutAttachments = new HashMap<String, Object>();
				wiedervorlageWithoutAttachments.putAll(WiedervorlageTestFactory.createAsMap());
				wiedervorlageWithoutAttachments.remove(WiedervorlageMapper.ATTACHMENTS);

				var wiedervorlage = mapper.fromItemMap(wiedervorlageWithoutAttachments, WiedervorlageTestFactory.VERSION);

				assertThat(wiedervorlage.getAttachments()).isEmpty();
			}

			@Test
			void shouldMapSingleAttachment() {
				var wiedervorlage = map();

				assertThat(wiedervorlage.getAttachments()).contains(BinaryFileTestFactory.FILE_ID);
			}

			@Test
			void shouldMapAttachmentList() {
				var wiedervorlage = mapper.fromItemMap(createWiedervorlageAsMapWithMultipleAttachments(), WiedervorlageTestFactory.VERSION);

				assertThat(wiedervorlage.getAttachments()).hasSize(2);
				assertThat(wiedervorlage.getAttachments().get(0)).isEqualTo(BinaryFileTestFactory.FILE_ID);
				assertThat(wiedervorlage.getAttachments().get(0)).isEqualTo(BinaryFileTestFactory.FILE_ID);
			}

			private HashMap<String, Object> createWiedervorlageAsMapWithMultipleAttachments() {
				var wiedervorlageItem = new HashMap<String, Object>();
				wiedervorlageItem.putAll(WiedervorlageTestFactory.createAsMap());
				wiedervorlageItem.put(WiedervorlageMapper.ATTACHMENTS, List.of(BinaryFileTestFactory.ID, BinaryFileTestFactory.ID));
				return wiedervorlageItem;
			}
		}

		private Wiedervorlage map() {
			return mapper.fromItemMap(WiedervorlageTestFactory.createAsMap(), WiedervorlageTestFactory.VERSION);
		}
	}
}